#include "InputTable.h"

#include "Config.h"

#include <boost/multi_index_container.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/multi_index/ordered_index.hpp>


using boost::multi_index_container;
using namespace boost::multi_index;

namespace Core
{

class Command2Key
{
public:
  Input::Command    m_command;
  std::string       m_name;
  int               m_keyCode;
  bool              m_state;

  Command2Key( Input::Command _command, std::string _name, int _keyCode ) : 
    m_command ( _command ),
    m_name    ( _name ),
    m_keyCode ( _keyCode ),
    m_state   ( false )
  {
  }
};

class Command2Key_ChangeState
{
public:
  Command2Key_ChangeState( const bool _newState ) : m_newState( _newState ) {}
  void operator() ( Command2Key& _c2k )
  {
    _c2k.m_state = m_newState;
  }
private:
  bool m_newState;
};


struct Command{};
struct Name{};
struct KeyCode{};

typedef multi_index_container<
  Command2Key,
  indexed_by<
    ordered_unique<
      tag<Command>,
        BOOST_MULTI_INDEX_MEMBER( Command2Key, Input::Command, m_command )
    >,
    ordered_unique<
      tag<Name>,BOOST_MULTI_INDEX_MEMBER( Command2Key, std::string, m_name )
    >,
    ordered_non_unique<
      tag<KeyCode>, BOOST_MULTI_INDEX_MEMBER( Command2Key, int, m_keyCode )
    >
  >
> Command2KeyList;

Command2KeyList&
command2KeyList()
{
  static Command2KeyList s_command2KeyList;
  return s_command2KeyList;
}


}; //Core

using namespace Core;


InputTable::InputTable(void)
{
  command2KeyList().insert( Command2Key( Input::Command::UP,            "up",             Config::get<unsigned>( "input_1.up" )       ) );
  command2KeyList().insert( Command2Key( Input::Command::DOWN,          "down",           Config::get<unsigned>( "input_1.down" )     ) );
  command2KeyList().insert( Command2Key( Input::Command::LEFT,          "left",           Config::get<unsigned>( "input_1.left" )     ) );
  command2KeyList().insert( Command2Key( Input::Command::RIGHT,         "right",          Config::get<unsigned>( "input_1.right" )    ) );
  command2KeyList().insert( Command2Key( Input::Command::ACTION_1,      "action_1",       Config::get<unsigned>( "input_1.action_1" ) ) );
  command2KeyList().insert( Command2Key( Input::Command::ACTION_2,      "action_2",       Config::get<unsigned>( "input_1.action_2" ) ) );
  command2KeyList().insert( Command2Key( Input::Command::NO,            "no",             Config::get<unsigned>( "input_1.no" )       ) );
  command2KeyList().insert( Command2Key( Input::Command::YES,           "yes",            Config::get<unsigned>( "input_1.yes" )      ) );
  command2KeyList().insert( Command2Key( Input::Command::ESCAPE,        "escape",         Config::get<unsigned>( "input_1.escape" )   ) );
  command2KeyList().insert( Command2Key( Input::Command::ENTER,         "enter",          Config::get<unsigned>( "input_1.enter" )    ) );
  command2KeyList().insert( Command2Key( Input::Command::STATS,         "stats",          Config::get<unsigned>( "input_1.stats" )    ) );
}

InputTable::~InputTable(void)
{
}

CommandState
getCommandState( const bool _oldState, bool _newState )
{
  CommandState returnState = CommandState::INVALID;

  if( _oldState )
  {
    if( _newState )
    {
      returnState = CommandState::ACTIVE;
    }
    else
    {
      returnState = CommandState::RELEASED;
    }
  }
  else
  {
    if( _newState )
    {
      returnState = CommandState::PRESSED;
    }
    else
    {
      returnState = CommandState::INACTIVE;
    }
  }
  return returnState;
}

void
InputTable::getCommandStates( CommandStates& _commandStates )
{
  _commandStates.clear();

  for(Command2KeyList::iterator it = command2KeyList().begin(); it != command2KeyList().end(); ++it)
  {
    const bool newState = Input::isKeyPressed( it->m_keyCode ) != 0;
    const bool oldState = it->m_state;
    
    command2KeyList().modify( it, Command2Key_ChangeState( newState ) );

    _commandStates[ it->m_command ] = getCommandState( oldState, newState );
  }
}

