#ifndef CORE_BINARYSERIALIZER_H_
#define CORE_BINARYSERIALIZER_H_

#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>

#include <boost/iostreams/device/back_inserter.hpp>
#include <boost/iostreams/filtering_stream.hpp>
#include <boost/iostreams/stream.hpp> 
#include <boost/iostreams/stream_buffer.hpp>
#include <boost/serialization/vector.hpp> 

//#define _VECSERIAL

#ifdef _VECSERIAL

#include <vector>
#include <algorithm>



namespace Core
{
typedef char                              SerializeBufferElem;
typedef std::vector<SerializeBufferElem>  SerializeBuffer;


class BinarySerializer : public boost::archive::binary_oarchive
{
private:
  SerializeBuffer* m_buffer;
public: 
  BinarySerializer( SerializeBuffer& _buffer ) : m_buffer( &_buffer ),
    boost::archive::binary_oarchive(
      boost::iostreams::stream<
        boost::iostreams::back_insert_device< SerializeBuffer >
      >( _buffer )
    )
  {
  }
  const SerializeBuffer& buffer() { return *m_buffer; }
};

class BinaryDeserializer : public boost::archive::binary_iarchive
{
private:
  SerializeBuffer* m_buffer;
public: 
  BinaryDeserializer( SerializeBuffer& _buffer ) : m_buffer( &_buffer ),
    boost::archive::binary_iarchive(
      boost::iostreams::stream<
        boost::iostreams::basic_array_source <SerializeBufferElem>
      >(
        boost::iostreams::basic_array_source<SerializeBufferElem>( &_buffer[0], _buffer.size() )
      )
    )
  {
  }

  SerializeBuffer& buffer() { return *m_buffer; }
};

}; //Core

#include "raknet/BitStream.h"

RakNet::BitStream& operator << (RakNet::BitStream& out, Core::BinarySerializer& in);
RakNet::BitStream& operator >> (RakNet::BitStream& in,  Core::BinaryDeserializer& out);

#else

#include <sstream>

namespace Core
{

typedef std::stringstream  SerializeBuffer;

class BinarySerializer : public boost::archive::binary_oarchive
{
private:
public: 
  SerializeBuffer* m_buffer;
  BinarySerializer( std::stringstream& _ss ) : m_buffer( &_ss ), boost::archive::binary_oarchive( _ss )
  {
  }

  const char*    data()  { return m_buffer->str().c_str(); }
  const unsigned size()  { return m_buffer->str().length(); }
  std::string    str()   { return m_buffer->str(); }
  void           flush() { m_buffer->flush(); }
};


class BinaryDeserializer : public boost::archive::binary_iarchive
{
private:
  SerializeBuffer* m_buffer;
public: 
  BinaryDeserializer( std::stringstream& _ss ) : m_buffer( &_ss ), boost::archive::binary_iarchive( _ss )
  {
  }

  const char*   data()                          { return m_buffer->str().data(); }
  std::string   str()                           { return m_buffer->str(); }
  void          str ( const std::string _str )  { m_buffer->str( _str ); }
};

}; //Core

#include "raknet/BitStream.h"

RakNet::BitStream& operator << (RakNet::BitStream& out, Core::BinarySerializer& in);
RakNet::BitStream& operator >> (RakNet::BitStream& in,  Core::BinaryDeserializer& out);
RakNet::BitStream& operator >> (RakNet::BitStream& in,  std::stringstream& out);

#endif // _VEVSERIAL

#endif //CORE_BINARYSERIALIZER_H_
