#pragma once

#include <blitz/tinyvec.h>


template < typename T >
class Vector2d : public blitz::TinyVector < T, 2 >
{
  public:
    Vector2d(){};
    Vector2d( const T& _x, const T& _y) : blitz::TinyVector < T, 2 >( _x, _y ){};

    template < typename T, template <typename> class U>
    Vector2d( const Vector2d<U>& _other) : Vector2d( static_cast<T>( _other.X() ), static_cast<T>( _other.Y() ) ) {};

    template<class Serializer>
    void serialize( Serializer& _s, const unsigned int _version )
    {
      _s & x();
      _s & y();
    }


    const T&  x() const { return blitz::TinyVector< T, 2 >::operator[]( 0 ); }
    T&        x()       { return blitz::TinyVector< T, 2 >::operator[]( 0 ); }
    const T&  y() const { return blitz::TinyVector< T, 2 >::operator[]( 1 ); }
    T&        y()       { return blitz::TinyVector< T, 2 >::operator[]( 1 ); }
};

template < typename T >
std::ostream& operator<< ( std::ostream& _stream, const Vector2d<T> _vec)
{
  _stream << _vec.x();
  _stream << _vec.y();
  return _stream;
}

template < typename T >
std::istream& operator>> ( std::istream& _stream, Vector2d<T> _vec)
{
  _stream >> _vec.x();
  _stream >> _vec.y();
  return _stream;
}

typedef Vector2d<unsigned>  Unsigned2d;
typedef Vector2d<int>       Int2d;
typedef Vector2d<float>     Float2d;
