#include "Creature.h"
#include "data/brain/Brain.h"
#include "data/world/World.h"
#include <cmath> //abs

using namespace Data;
using Core::Direction;

Network::ReplicaFactoryRegister<Creature> Creature::replicaFactoryRegister;

void
Creature::update( const float& _secs )
{
  if( m_brain )
  {
    m_brain->update( _secs );
  }

  if( m_speed > 0.01 )
  {
    const float length = m_speed * _secs;
    move( length );
  }
}

void
moveFixOffset( float& refOffset, unsigned& refTile )
{
  const float absOffset = abs( refOffset );

  if( absOffset > 0.5f )
  {
    const float tileOffset = ceil( absOffset - 0.5f );

    if( refOffset > 0.0f )
    {
      refTile   += static_cast<unsigned>( tileOffset );
      refOffset -= tileOffset;
    }
    else
    {
      refTile   -= static_cast<unsigned>( tileOffset );
      refOffset += tileOffset;
    }
  }
}

void
Creature::move( const float& _length )
{
  Tiles::Tile currTile = World::tiles().tile( tile() );

  if( currTile.isInvalid() )
  {
    respawn();
  }

  if( World::tiles().isWalkable( tile(), offset(), direction() ) )
  {
    Int2d dir;
    dir = direction().toOffset();

    m_offset.x() += static_cast<float>( dir.x() ) * _length;
    m_offset.y() += static_cast<float>( dir.y() ) * _length;

    moveFixOffset( m_offset.y(), m_tile.y() );
    moveFixOffset( m_offset.x(), m_tile.x() );
  }
  else
  {
    speed( 0.0f );
  }
}

void
Creature::respawn()
{
  m_tile = Unsigned2d( 1, 1 );
  m_offset = Float2d( 0.0f, 0.0f );
}

////////////////////
// DE/INIT
Creature::Creature(void) :
  m_direction( Direction::UP ),
  m_tile    ( 1, 1 ),
  m_offset  ( 0.0f, 0.0f ),
  m_speed   ( 0.0f ),
  m_brain   ( 0 ),
  m_maxSpeed( 2.0f )
{
  eventCreateCreature().notify( this );
}

Creature::~Creature(void)
{
  eventDestroyCreature().notify( this );
}
// DE/INIT
////////////////////

////////////////////
// GET/SET

void
Creature::direction ( const Direction& _dir )
{
  m_direction = _dir;
}

Direction&
Creature::direction ()
{
  return m_direction;
}

const Direction&
Creature::direction () const
{
  return m_direction;
}

void
Creature::speed ( const float& _speed )
{
  m_speed = _speed;
}

const float
Creature::speed () const
{
  return m_speed;
}

void
Creature::setToMaxSpeed()
{
  speed( m_maxSpeed );
}

void
Creature::brain ( Brain* _brain )
{
  if( m_brain )
  {
    delete m_brain;
  }

  m_brain = _brain;
}

// GET/SET
////////////////////

////////////////////
// SIGNALS

Core::ObserverSubject< Creature* const >&
Creature::eventCreateCreature()
{
  static Core::ObserverSubject< Creature* const >  s_eventCreateCreature;
  return s_eventCreateCreature;
}

Core::ObserverSubject< Creature* const >&
Creature::eventDestroyCreature()
{
  static Core::ObserverSubject< Creature* const >  s_eventDestroyCreature;
  return s_eventDestroyCreature;
}

// SIGNALS
////////////////////

