#pragma once

#include "Config.h"

#include "data/type/Type.h"

#include "network/replica/ReplicaFactory.h"

#include "core/pattern/Observer.h"
#include "core/types/Direction.h"
#include "core/types/Vector2d.h"

namespace Data {

class Brain;

class Creature : public Network::Replica
{
private:
  ///////////////
  // DYNAMIC
  Core::Direction m_direction;
  Unsigned2d  m_tile;
  Float2d   m_offset;
  float     m_speed;

  ///////////////
  // LOCAL
  Brain*    m_brain;
  float     m_maxSpeed;

public:
  Creature(void);
  ~Creature(void);

  void update( const float& _secs );

  void                      direction ( const Core::Direction& _dir );
  Core::Direction&          direction ();
  const Core::Direction&    direction () const;

  void                      speed ( const float& _speed );
  const float               speed () const;
  void                      setToMaxSpeed ();

  const Unsigned2d          tile()  const { return m_tile; }

  const Float2d             offset() const { return m_offset; }

  void                      brain ( Brain* _brain );

  void                      respawn();

private:
  void                      move( const float& _length );

///////////////////
// Type
public:
  const Type          type() const    { return staticType(); };
  static  const Type  staticType()    { return Type::getCreature(); }

///////////////////
// Events
public:
  static Core::ObserverSubject< Creature* const >& eventCreateCreature();
  static Core::ObserverSubject< Creature* const >& eventDestroyCreature();

///////////////////
// Replica
public:
  virtual unsigned    getType() const { return staticType(); };

  template<class Serializer>
  void serialize( Serializer& _s, const unsigned int _version )
  {
    _s & m_speed;
    _s & m_tile;
    _s & m_offset;
    _s & m_direction;
  }
protected:
  
  virtual void serialize  ( Core::BinarySerializer& refOutput )  { serialize( refOutput, 1 ); }
  virtual void deserialize( Core::BinaryDeserializer& refInput ) { serialize( refInput,  1 ); }

public:


  static  Network::Replica* create()
  {
    return new Creature();
  };

  static Network::ReplicaFactoryRegister<Creature> replicaFactoryRegister;

};

}; //Data