#include "Tiles.h"

#include "Config.h"

using namespace Data;

Tiles::Tiles(void)
{
}

Tiles::~Tiles(void)
{
}

void
Tiles::init()
{
  m_tiles.clear();
  eventCreateTiles().notify( this );
}

void
Tiles::deinit()
{
  eventDestroyTiles().notify( this );
  m_tiles.clear();
}

void
Tiles::resize( const unsigned _x, const unsigned _y)
{
  const unsigned oldX = m_tiles.col.size();
  const unsigned oldY = m_tiles.row.size();

  //Old Borders
  for ( unsigned i = 0; i < m_tiles.col.size(); ++i )
  {
    m_tiles( 0,i ) = Tile::FREE;
    m_tiles( m_tiles.row.size() - 1, i ) = Tile::FREE;
  }

  for ( unsigned i = 0; i < m_tiles.row.size(); ++i )
  {
    m_tiles( i, 0 ) = Tile::FREE;
    m_tiles( i, m_tiles.col.size() - 1 ) = Tile::FREE;
  }

  m_tiles.resize( _x, _y, Tile::FREE );

  //New Borders
  for ( unsigned i = 0; i < m_tiles.col.size(); ++i )
  {
    m_tiles( 0,i ) = Tile::SOLID;
    m_tiles( m_tiles.row.size() - 1, i ) = Tile::SOLID;
  }

  for ( unsigned i = 0; i < m_tiles.row.size(); ++i )
  {
    m_tiles( i, 0 ) = Tile::SOLID;
    m_tiles( i, m_tiles.col.size() - 1 ) = Tile::SOLID;
  }

  eventUpdateTiles().notify( this );
}

const Tiles::Tile
Tiles::tile( const Unsigned2d& _pos ) const
{
  if( _pos.x() >= m_tiles.row.size() || _pos.y() >= m_tiles.col.size() )
  {
    return Tile::INVALID;
  }
  return m_tiles( _pos.x(), _pos.y() );
}

void 
Tiles::setTile( const Unsigned2d& _pos, Tile _tile )
{
  if( _pos.x() >= m_tiles.row.size() || _pos.y() >= m_tiles.col.size() )
  {
    return;
  }
  m_tiles( _pos.x(), _pos.y() ) = _tile;
  eventUpdateTiles().notify( this );
}

const bool
Tiles::isWalkable( const Unsigned2d& _pos, const Float2d& _offset, const Core::Direction _dir ) const
{

  const float innerOffsetLimit = 0.1f;
  const float outerOffsetLimit = 0.2f;

  //////////////////////////
  // GET TILES OF INTEREST
  Int2d primaryOffset = _dir.toOffset();

  if( primaryOffset.x() && ( primaryOffset.x() * _offset.x() < innerOffsetLimit ) )
  {
    primaryOffset.x() = 0;
  }

  if( primaryOffset.y() && ( primaryOffset.y() * _offset.y() < innerOffsetLimit ) )
  {
    primaryOffset.y() = 0;
  }

  Unsigned2d  primaryGoal( _pos.x() + primaryOffset.x(), _pos.y() + primaryOffset.y() );

  Int2d secondaryOffset = primaryOffset;

  if( primaryOffset.y() && abs( _offset.x() ) > outerOffsetLimit ) 
  {
    secondaryOffset.x() += ( _offset.x() > 0.0f ) ? 1 : -1 ;
  }

  if( primaryOffset.x() && abs( _offset.y() ) > outerOffsetLimit )
  {
    secondaryOffset.y() += ( _offset.y() > 0.0f ) ? 1 : -1 ;
  }

  Unsigned2d  secondaryGoal( _pos.x() + secondaryOffset.x(), _pos.y() + secondaryOffset.y() );
  // GET TILES OF INTEREST
  //////////////////////////

  bool isWalkable( false );

  isWalkable  = tile( primaryGoal ).isWalkable();
  isWalkable &= tile( secondaryGoal ).isWalkable();

  return isWalkable ;
}

void
Tiles::update( const float _secs )
{
}

Core::ObserverSubject< const Tiles* >&
Tiles::eventCreateTiles()
{
  static Core::ObserverSubject< const Tiles* >  s_eventCreateTiles;
  return s_eventCreateTiles;
}

Core::ObserverSubject< const Tiles* >&
Tiles::eventDestroyTiles()
{
  static Core::ObserverSubject< const Tiles* >  s_eventDestroyTiles;
  return s_eventDestroyTiles;
}

Core::ObserverSubject< const Tiles* >&
Tiles::eventUpdateTiles()
{
  static Core::ObserverSubject< const Tiles* >  s_eventUpdateTiles;
  return s_eventUpdateTiles;
}
