#pragma once

#include "core/pattern/Observer.h"

#include "core/types/Matrix.h"
#include "core/types/Vector2d.h"
#include "core/types/Direction.h"

#include <string>

namespace Data {

class Tiles
{
public:
    class Tile
    {
      public:
        enum eTile
        {
          INVALID = 0,
          FREE,
          ROCK,
          SOLID
        };

        Tile( void ) : m_value( INVALID ) {}
        Tile( eTile _tile ) : m_value( _tile ) {}

        const eTile& value() const    { return m_value; };
        eTile& value()                { return m_value; };

        const bool isWalkable() const { return ( m_value == FREE ); };
        const bool isBlocked() const  { return ( m_value != FREE ); };
        const bool isInvalid() const  { return ( m_value == INVALID ); };
        const bool isFree() const     { return ( m_value == FREE ); };
        const bool isRock() const     { return ( m_value == ROCK ); };
        const bool isSolid() const    { return ( m_value == SOLID ); };

      private:
        eTile m_value;
    };
  private:
    typedef YMatrix::Matrix<Tile, YMatrix::DenseMatrix, YMatrix::NonMathMatrix> TileMatrix;
  public:
    Tiles(void);
    ~Tiles(void);

    void init();
    void deinit();

    const unsigned  width()  const { return m_tiles.row.size(); }
    const unsigned  height() const { return m_tiles.col.size(); }
    const bool      isWalkable( const Unsigned2d& _pos, const Float2d& _offset, const Core::Direction _dir ) const;
    const Tile      tile( const Unsigned2d& _pos ) const;
  protected:
    friend class World;
    void            setTile( const Unsigned2d& _pos, Tile _tile );

  public:
    void            resize( const unsigned _x, const unsigned _y);
    void            update( const float _secs );

  public:
    static Core::ObserverSubject< const Tiles* >& eventCreateTiles();
    static Core::ObserverSubject< const Tiles* >& eventDestroyTiles();
    static Core::ObserverSubject< const Tiles* >& eventUpdateTiles();

  private:
    TileMatrix m_tiles;

};

}; //Data