#pragma once

#include <iostream>

#include "ReplicaManager.h"

#include "core/log/Log.h"
#include "core/serializer/BinarySerializer.h"

namespace Network {


class Replica : public RakNet::Replica3
{
///////////////
// our system
public:

  virtual unsigned getType() const = 0;

protected:
  
  virtual void serialize  ( Core::BinarySerializer& refOutput )  = 0;
  virtual void deserialize( Core::BinaryDeserializer& refInput ) = 0;

public:

  static ReplicaManager& replicaManager()
  {
    static ReplicaManager s_replicaManager;
    return s_replicaManager;
  }

  Replica::Replica()
  {
//    replicaManager().Reference( this );
  }

///////////////
// foreign system
public:
	//virtual RakNet::RakString getName(void) const = 0;

	virtual void WriteAllocationID(RakNet::BitStream *allocationIdBitstream) const
  {
		allocationIdBitstream->Write( getType() );
	}

  virtual void SerializeConstruction(RakNet::BitStream *constructionBitstream, RakNet::Connection_RM3 *destinationConnection)
  {
    LOG_DEBUG1 << "Replica::SerializeConstruction\n";
  }

	virtual bool DeserializeConstruction(RakNet::BitStream *constructionBitstream, RakNet::Connection_RM3 *sourceConnection)
  {
    LOG_DEBUG1 << "Replica::DeserializeConstruction\n";
		return true;
	}

	virtual void SerializeDestruction(RakNet::BitStream *destructionBitstream, RakNet::Connection_RM3 *destinationConnection)
  {
    LOG_DEBUG1 << "Replica::SerializeDestruction\n";
	}

	virtual bool DeserializeDestruction(RakNet::BitStream *destructionBitstream, RakNet::Connection_RM3 *sourceConnection)
  {
    LOG_DEBUG1 << "Replica::DeserializeDestruction\n";
		return true;
	}

  virtual void DeallocReplica(RakNet::Connection_RM3 *sourceConnection)
  {
    LOG_DEBUG1 << "Replica::DeallocReplica\n";
		delete this;
	}

	virtual RakNet::RM3SerializationResult Serialize( RakNet::SerializeParameters *serializeParameters )
  {
    LOG_DEBUG1 << "Replica::Serialize\n";

    Core::SerializeBuffer buffer( std::ios::out | std::ios::binary );
    Core::BinarySerializer serializer( buffer );

    serialize( serializer );

    serializeParameters->outputBitstream[0] << serializer;

    return RakNet::RM3SR_SERIALIZED_ALWAYS;
	}

	virtual void Deserialize( RakNet::DeserializeParameters *deserializeParameters )
  {
    LOG_DEBUG1 << "Replica::Deserialize\n";

    Core::SerializeBuffer buffer( std::ios::in | std::ios::binary );

    deserializeParameters->serializationBitstream[0] >> buffer;

    Core::BinaryDeserializer deserializer( buffer );

    deserialize( deserializer );
	}

	virtual RakNet::RM3ConstructionState QueryConstruction(RakNet::Connection_RM3 *destinationConnection, RakNet::ReplicaManager3 *replicaManager3)
  {
		return QueryConstruction_PeerToPeer(destinationConnection);
	}

	virtual bool QueryRemoteConstruction(RakNet::Connection_RM3 *sourceConnection)
  {
		return QueryRemoteConstruction_PeerToPeer(sourceConnection);
	}

	virtual RakNet::RM3QuerySerializationResult QuerySerialization(RakNet::Connection_RM3 *destinationConnection)
  {
		return QuerySerialization_PeerToPeer(destinationConnection);
	}

	virtual RakNet::RM3ActionOnPopConnection QueryActionOnPopConnection(RakNet::Connection_RM3 *droppedConnection) const
  {
		return QueryActionOnPopConnection_PeerToPeer(droppedConnection);
	}

};

}; //Network