#include "Creature.h"

#include "Config.h"

#include "data/entity/Creature.h"

#include "core/ressources/Ressources.h"

#include <map>

using Core::Ressources;
using Core::Direction;

namespace Render
{
  typedef std::map< const Data::Creature* const, Creature* > CreatureDataRenderRelation;
  CreatureDataRenderRelation s_creatureRelation;
};

namespace Render
{

class CreateCreatureObserver : public Core::Observer< Data::Creature* const >
{
  public:
    CreateCreatureObserver()
    {
      Data::Creature::eventCreateCreature().add(this);
    }

    virtual void notify( Data::Creature* const _data ) 
    {
      new Creature( _data );
    }
} creatureCreateObserver;

class DestroyCreatureObserver : public Core::Observer< Data::Creature* const >
{
  public:
    DestroyCreatureObserver()
    {
      Data::Creature::eventDestroyCreature().add(this);
    }

    virtual void notify( Data::Creature* const _data ) 
    {
      Creature* render = Creature::getRenderByData( _data );
      if( render )
      {
        delete( render );
      }
    }
} creatureDestroyObserver;

}; // Render

using namespace Render;

Creature* 
Creature::getRenderByData( const Data::Creature* _data )
{
  CreatureDataRenderRelation::const_iterator it = s_creatureRelation.find( _data );

  if( it != s_creatureRelation.end() )
  {
    return s_creatureRelation[_data];
  }
  
  return 0;
}

Creature::Creature( const Data::Creature* _data ) :
  m_sprite( Ressources::Sprite::BOMBERMAN ),
  m_data( _data )
{
  s_creatureRelation[_data] = this;
  
  updatePosition();

  m_sprite.show();
}

Creature::~Creature(void)
{
  m_data = 0;

  CreatureDataRenderRelation::const_iterator it = s_creatureRelation.find( m_data );

  if( it != s_creatureRelation.end() )
  {
    s_creatureRelation.erase( it );
  }

}

void
Creature::animateCreature() const
{
  unsigned rowOffset ( 1 );

  switch( m_data->direction().value() )
  {
    case Direction::UP     : rowOffset = 34; break;
    case Direction::DOWN   : rowOffset = 1; break;
    case Direction::LEFT   : rowOffset = 12; break;
    case Direction::RIGHT  : rowOffset = 23; break;
  }

  unsigned colOffsetA( rowOffset + 0 );
  unsigned colOffsetB( rowOffset + 2 );

  int frameDelay = 200;


  if( m_data->speed() > 0.01 )
  {
    colOffsetA = rowOffset + 3;
    colOffsetB = rowOffset + 10;
    frameDelay = static_cast<int>( 100.0f / m_data->speed() );
  }

  m_sprite.animate( colOffsetA, colOffsetB, frameDelay );
}

void
Creature::updatePosition() const
{
  static const int tileSize = Config::get<unsigned>( "sprite.tileSize" );

  Int2d pos;

  pos.x() = m_data->tile().x() * tileSize;
  pos.y() = m_data->tile().y() * tileSize;

  pos.x() += static_cast<int>( m_data->offset().x() * tileSize );
  pos.y() += static_cast<int>( m_data->offset().y() * tileSize );

  m_sprite.setPosition( pos );
}

void
Creature::draw( const float& _secs )
{
  updatePosition();
  animateCreature();
}

